//
//  PluginLibrary.mm
//  TemplateApp
//
//  Copyright (c) 2012 __MyCompanyName__. All rights reserved.
//

#import "GameAnalyticsPlugin.h"

#include <CoronaRuntime.h>
#import <UIKit/UIKit.h>

#import <GameAnalytics/GameAnalytics.h>
// ----------------------------------------------------------------------------

class GameAnalyticsPlugin
{
	public:
		typedef GameAnalyticsPlugin Self;

	public:
		static const char kName[];
		static const char kEvent[];

	protected:
		GameAnalyticsPlugin();

	public:
		bool Initialize( CoronaLuaRef listener );

	public:
		CoronaLuaRef GetListener() const { return fListener; }

	public:
		static int Open( lua_State *L );

	protected:
		static int Finalizer( lua_State *L );

	public:
		static Self *ToLibrary( lua_State *L );

	public:
		static int init( lua_State *L );
static int addImpressionMaxEvent( lua_State *L );
static int setEnabledInfoLog( lua_State *L );
static int configureBuild( lua_State *L );
static int initialize( lua_State *L );
static int addDesignEvent( lua_State *L );
static int addProgressionEvent( lua_State *L );

	private:
		CoronaLuaRef fListener;
};

// ----------------------------------------------------------------------------

// This corresponds to the name of the library, e.g. [Lua] require "plugin.library"
const char GameAnalyticsPlugin::kName[] = "plugin.gameanalytics_playza";

// This corresponds to the event name, e.g. [Lua] event.name
const char GameAnalyticsPlugin::kEvent[] = "pluginlibraryevent";

GameAnalyticsPlugin::GameAnalyticsPlugin()
:	fListener( NULL )
{
}

bool
GameAnalyticsPlugin::Initialize( CoronaLuaRef listener )
{
	// Can only initialize listener once
	bool result = ( NULL == fListener );

	if ( result )
	{
		fListener = listener;
	}

	return result;
}

int
GameAnalyticsPlugin::Open( lua_State *L )
{
	// Register __gc callback
	const char kMetatableName[] = __FILE__; // Globally unique string to prevent collision
	CoronaLuaInitializeGCMetatable( L, kMetatableName, Finalizer );

	// Functions in library
	const luaL_Reg kVTable[] =
	{
		{ "init", init },
  {"addImpressionMaxEvent", addImpressionMaxEvent},
  {"setEnabledInfoLog", setEnabledInfoLog},
  {"configureBuild", configureBuild},
  {"initialize",initialize},
  {"addDesignEvent",addDesignEvent},
  {"addProgressionEvent",addProgressionEvent},

		{ NULL, NULL }
	};

	// Set library as upvalue for each library function
	Self *library = new Self;
	CoronaLuaPushUserdata( L, library, kMetatableName );

	luaL_openlib( L, kName, kVTable, 1 ); // leave "library" on top of stack

	return 1;
}

int
GameAnalyticsPlugin::Finalizer( lua_State *L )
{
	Self *library = (Self *)CoronaLuaToUserdata( L, 1 );

	CoronaLuaDeleteRef( L, library->GetListener() );

	delete library;

	return 0;
}

GameAnalyticsPlugin *
GameAnalyticsPlugin::ToLibrary( lua_State *L )
{
	// library is pushed as part of the closure
	Self *library = (Self *)CoronaLuaToUserdata( L, lua_upvalueindex( 1 ) );
	return library;
}

// [Lua] library.init( listener )
int
GameAnalyticsPlugin::init( lua_State *L )
{
	int listenerIndex = 1;

	if ( CoronaLuaIsListener( L, listenerIndex, kEvent ) )
	{
		Self *library = ToLibrary( L );

		CoronaLuaRef listener = CoronaLuaNewRef( L, listenerIndex );
		library->Initialize( listener );
	}

	return 0;
}

// [Lua] init

int
GameAnalyticsPlugin::initialize(lua_State *L)
{

    Self *context = ToLibrary(L);
  
  if (! context) { // abort if no valid context
    return 0;
  }
  
//  Self& library = *context;
  
    if (lua_gettop(L) < 1 || !lua_istable(L, 1)) {
        return 0;
    }

    lua_getfield(L, 1, "gameKey");
    lua_getfield(L, 1, "gameSecret");

    if (!lua_isstring(L, -2) || !lua_isstring(L, -1)) {
        return 0;
    }

    const char *gameKey = lua_tostring(L, -2);
    const char *gameSecret = lua_tostring(L, -1);
    lua_pop(L, 2);

    NSString *nsGameKey = [NSString stringWithUTF8String:gameKey];
    NSString *nsGameSecret = [NSString stringWithUTF8String:gameSecret];

    dispatch_async(dispatch_get_main_queue(), ^{
        [GameAnalytics initializeWithGameKey:nsGameKey gameSecret:nsGameSecret];
    });
    return 0;
}

int GameAnalyticsPlugin::configureBuild(lua_State *L)
{
    if (lua_gettop(L) < 1 || !lua_isstring(L, 1)) {
        return 0;
    }

    const char *buildString = lua_tostring(L, 1);
    NSString *nsBuildString = [NSString stringWithUTF8String:buildString];

    dispatch_async(dispatch_get_main_queue(), ^{
        [GameAnalytics configureBuild:nsBuildString];
    });

    return 1;
}

int GameAnalyticsPlugin::setEnabledInfoLog(lua_State *L) {
    if (lua_gettop(L) < 1 || !lua_isboolean(L, 1)) {
        return 0;
    }

    BOOL value = lua_toboolean(L, 1);

    dispatch_async(dispatch_get_main_queue(), ^{
        [GameAnalytics setEnabledInfoLog:value];
        [GameAnalytics setEnabledVerboseLog:value];
    });

    return 1;
}

int GameAnalyticsPlugin::addImpressionMaxEvent(lua_State *L) {
    if (lua_gettop(L) < 2 || !lua_isstring(L, 1) || !lua_istable(L, 2)) {
        return 0;
    }

    NSString *version = [NSString stringWithUTF8String: lua_tostring(L, 1)];
    NSMutableDictionary *json = [NSMutableDictionary dictionary];

    lua_getfield(L, 2, "country");
    if (lua_isstring(L, -1)) {
        json[@"country"] = [NSString stringWithUTF8String:lua_tostring(L, -1)];
    }
    lua_pop(L, 1);

    lua_getfield(L, 2, "network_name");
    if (lua_isstring(L, -1)) {
        json[@"network_name"] = [NSString stringWithUTF8String:lua_tostring(L, -1)];
    }
    lua_pop(L, 1);

    lua_getfield(L, 2, "adunit_id");
    if (lua_isstring(L, -1)) {
        json[@"adunit_id"] = [NSString stringWithUTF8String:lua_tostring(L, -1)];
    }
    lua_pop(L, 1);

    lua_getfield(L, 2, "adunit_format");
    if (lua_isstring(L, -1)) {
        NSString *format = @"";
        NSString *d = [NSString stringWithUTF8String:lua_tostring(L, -1)];
       
        if ([d rangeOfString:@"banner" options:NSCaseInsensitiveSearch].location != NSNotFound){
            format = @"BANNER";
        }
        else if ([d rangeOfString:@"INTER" options:NSCaseInsensitiveSearch].location != NSNotFound){
            format = @"INTER";
            }
        else if ([d rangeOfString:@"REWARDED" options:NSCaseInsensitiveSearch].location != NSNotFound){
            format = @"REWARDED";
            }
        else
        {
        format = d;
        }
        json[@"adunit_format"] = format;
    }
    lua_pop(L, 1);

    lua_getfield(L, 2, "placement");
    if (lua_isstring(L, -1)) {
        json[@"placement"] = [NSString stringWithUTF8String:lua_tostring(L, -1)];
    }
    lua_pop(L, 1);

    lua_getfield(L, 2, "creative_id");
    if (lua_isstring(L, -1)) {
        json[@"creative_id"] = [NSString stringWithUTF8String:lua_tostring(L, -1)];
    }
    lua_pop(L, 1);

    lua_getfield(L, 2, "revenue");
    if (lua_isnumber(L, -1)) {
        json[@"revenue"] = @(lua_tonumber(L, -1));
    } else {
        json[@"revenue"] = @(0); // Значение по умолчанию
    }
    lua_pop(L, 1);

    dispatch_async(dispatch_get_main_queue(), ^{
        @try {
            [GameAnalytics addImpressionTopOnEventWithAdNetworkVersion:version impressionData:json];
            
        } @catch (NSException *exception) {
            NSLog(@"GameAnalytics.addImpressionMaxEvent failed: %@", exception);
        }
    });

    return 1;
}

int GameAnalyticsPlugin::addDesignEvent(lua_State *L) {
    if (lua_gettop(L) < 1 || !lua_istable(L, 1)) {
        return 0;
    }

    lua_getfield(L, 1, "eventId");
    if (!lua_isstring(L, -1)) {
        return 0;
    }
    const char *eventId = lua_tostring(L, -1);
    lua_pop(L, 1);

    lua_getfield(L, 1, "value");
    BOOL hasValue = lua_isnumber(L, -1);
    double value = hasValue ? lua_tonumber(L, -1) : 0;
    lua_pop(L, 1);

    dispatch_async(dispatch_get_main_queue(), ^{
        if (hasValue) {
            [GameAnalytics addDesignEventWithEventId:[NSString stringWithUTF8String:eventId] value:@(value)];
        } else {
            [GameAnalytics addDesignEventWithEventId:[NSString stringWithUTF8String:eventId]];
        }
    });

    return 1;
}


int GameAnalyticsPlugin::addProgressionEvent(lua_State *L) {
    if (lua_gettop(L) < 1 || !lua_istable(L, 1)) {
        return 0;
    }

    lua_getfield(L, 1, "progressionStatus");
    if (!lua_isstring(L, -1)) {
        return 0;
    }
    NSString *progressionStatusStr = [NSString stringWithUTF8String:lua_tostring(L, -1)];
    lua_pop(L, 1);

    GAProgressionStatus progressionStatus;
    if ([progressionStatusStr caseInsensitiveCompare:@"start"] == NSOrderedSame) {
        progressionStatus = GAProgressionStatusStart;
    } else if ([progressionStatusStr caseInsensitiveCompare:@"complete"] == NSOrderedSame) {
        progressionStatus = GAProgressionStatusComplete;
    } else if ([progressionStatusStr caseInsensitiveCompare:@"fail"] == NSOrderedSame) {
        progressionStatus = GAProgressionStatusFail;
    } else {
        return 0;
    }

    lua_getfield(L, 1, "progression01");
    lua_getfield(L, 1, "progression02");
    lua_getfield(L, 1, "progression03");

    if (!lua_isstring(L, -3)) {
        return 0;
    }

    NSString *progression01 = [NSString stringWithUTF8String:lua_tostring(L, -3)];
    NSString *progression02 = lua_isstring(L, -2) ? [NSString stringWithUTF8String:lua_tostring(L, -2)] : nil;
    NSString *progression03 = lua_isstring(L, -1) ? [NSString stringWithUTF8String:lua_tostring(L, -1)] : nil;

    lua_pop(L, 3);

    lua_getfield(L, 1, "score");
    BOOL hasScore = lua_isnumber(L, -1);
    int score = hasScore ? (int)lua_tointeger(L, -1) : 0;
    lua_pop(L, 1);

    dispatch_async(dispatch_get_main_queue(), ^{
        if (hasScore) {
            [GameAnalytics addProgressionEventWithProgressionStatus:progressionStatus
                                                      progression01:progression01
                                                      progression02:progression02
                                                      progression03:progression03
                                                              score:score];
        } else {
            [GameAnalytics addProgressionEventWithProgressionStatus:progressionStatus
                                                      progression01:progression01
                                                      progression02:progression02
                                                      progression03:progression03];
        }
    });

    return 1;
}
// ----------------------------------------------------------------------------

CORONA_EXPORT int luaopen_plugin_gameanalytics_playza( lua_State *L )
{
	return GameAnalyticsPlugin::Open( L );
}
